package com.sun.org.apache.bcel.internal.generic;

import com.sun.org.apache.bcel.internal.ExceptionConst;
import com.sun.org.apache.bcel.internal.classfile.ConstantPool;
import com.sun.org.apache.bcel.internal.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public class MULTIANEWARRAY extends CPInstruction implements LoadClass, AllocationInstruction, ExceptionThrower
{
    private short dimensions;
    
    MULTIANEWARRAY() {
    }
    
    public MULTIANEWARRAY(int index, short dimensions) {
        super((short)197, index);
        if (dimensions < 1) {
            throw new ClassGenException("Invalid dimensions value: " + (int)dimensions);
        }
        this.dimensions = dimensions;
        super.setLength(4);
        return;
    }
    
    @Override
    public void dump(DataOutputStream out) throws IOException {
        out.writeByte((int)super.getOpcode());
        out.writeShort(super.getIndex());
        out.writeByte((int)this.dimensions);
    }
    
    @Override
    protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
        super.initFromFile(bytes, wide);
        this.dimensions = (short)bytes.readByte();
        super.setLength(4);
    }
    
    public final short getDimensions() {
        return this.dimensions;
    }
    
    @Override
    public String toString(boolean verbose) {
        return super.toString(verbose) + " " + super.getIndex() + " " + (int)this.dimensions;
    }
    
    @Override
    public String toString(ConstantPool cp) {
        return super.toString(cp) + " " + (int)this.dimensions;
    }
    
    @Override
    public int consumeStack(ConstantPoolGen cpg) {
        return this.dimensions;
    }
    
    @Override
    public Class<?>[] getExceptions() {
        return ExceptionConst.createExceptions(ExceptionConst.EXCS.EXCS_CLASS_AND_INTERFACE_RESOLUTION, new Class[] { ExceptionConst.ILLEGAL_ACCESS_ERROR, ExceptionConst.NEGATIVE_ARRAY_SIZE_EXCEPTION });
    }
    
    @Override
    public ObjectType getLoadClassType(ConstantPoolGen cpg) {
        Type t = this.getType(cpg);
        if ((t instanceof ArrayType)) {
            t = ((ArrayType)t).getBasicType();
        }
        return (t instanceof ObjectType) ? ((ObjectType)t) : null;
    }
    
    @Override
    public void accept(Visitor v) {
        v.visitLoadClass(this);
        v.visitAllocationInstruction(this);
        v.visitExceptionThrower(this);
        v.visitTypedInstruction(this);
        v.visitCPInstruction(this);
        v.visitMULTIANEWARRAY(this);
    }
}
